# ecfdataimport.py
# Copyright 2013 Roger Marsh
# Licence: See LICENCE (BSD licence)

"""Extract player and club data from ECF reference files.

ECF (formerly BCF) provided reference files on CD from the end of the 1998
season as a matter of routine.  These contained the active players from the
most recent three seasons.  From the end of the 2006 season smaller updates
were provided during the season but these were soon replaced by more frequent
issuing of reference files including new, or re-activated, players.  From the
middle of the 2011 season the reference files included all players who had
ever played a graded game (since early 1990s in practice).

"""

from ..core import filespec
from ..core import ecfrecord
from ..core import ecfclubdb
from ..core import ecfplayerdb
from ..core import ecfmaprecord


def _strify(value):
    """Tranform a value from an ECF file to str.

    Assume iso-8859-1 encoding for value.

    """
    return value.decode('iso-8859-1')


def _utf8ify(value):
    """Tranform a value from an ECF file to utf-8 encoding."""
    return _strify(value)


def _coerce_to_utf8(value):
    """Tranform a str value to utf-8 encoding."""
    if not isinstance(value, bytes):
        return value
    else:
        return _strify(value)


def copy_ecf_clubs_post_1998_rules(
    results,
    logwidget=None,
    ecffile=None,
    ecfdate=None,
    parent=None,
    datecontrol=None,
    strify=_strify,
    utf8ify=_utf8ify,
    coerce_to_utf8=_coerce_to_utf8,
    **kwargs
    ):
    """Import a new ECF club file

    widget - the manager object for the ecf data import tab

    """
    txndaterec = ecfrecord.ECFrefDBrecordECFdate()
    txndaterec.value.ECFdate = ecfdate
    txndaterec.value.ECFtxntype = ecfrecord.txnNew
    txndaterec.value.ECFobjtype = ecfrecord.objtypeClub
    txndaterec.key.recno = 0
    daterecord = ecfrecord.ECFrefDBrecordECFdate()
    datecursor = results.database_cursor(
        filespec.ECFTXN_FILE_DEF, filespec.ECFDATE_FIELD_DEF)
    try:
        r = datecursor.nearest(coerce_to_utf8(ecfdate))
        while r:
            daterecord.load_instance(
                results,
                filespec.ECFTXN_FILE_DEF,
                filespec.ECFDATE_FIELD_DEF,
                r)
            if ecfdate != daterecord.value.ECFdate:
                break
            if txndaterec.value == daterecord.value:
                if logwidget:
                    logwidget.append_text('', timestamp=False)
                    logwidget.append_text(' '.join(
                        ['A Master club file for',
                         datecontrol,
                         'has been input previously.']))
                return False
            r = datecursor.next()
    finally:
        datecursor.close()
    results.start_transaction()
    txndaterec.put_record(results, filespec.ECFTXN_FILE_DEF)
    ecfrecord.ECFrefDBrecordECFdate.set_most_recent_master_dates(results)

    ecfrec = ecfrecord.ECFrefDBrecordECFclub()
    ecfimp = ecfclubdb.ECFclubsDBrecord()

    ecfcursor = results.database_cursor(
        filespec.ECFCLUB_FILE_DEF, filespec.ECFCLUBCODE_FIELD_DEF)
    try:
        clubs = ecffile.database_cursor(ecfclubdb.CLUBS, ecfclubdb.CLUBS)
    
        # Load the ECF data
        try:
            data = clubs.first()
            while data:
                ecfimp.load_instance(
                    ecffile, ecfclubdb.CLUBS, ecfclubdb.CLUBS, data)
                record = ecfcursor.nearest(utf8ify(ecfimp.value.CODE))
                if record == None:
                    ecfrec.key.recno = 0
                    ecfrec.value.ECFcode = strify(ecfimp.value.CODE)
                    ecfrec.value.ECFname = strify(ecfimp.value.CLUB)
                    ecfrec.value.ECFcountycode = strify(ecfimp.value.COUNTY)
                    ecfrec.value.calculate_ecf_active_club()
                    ecf_codes.add(ecfrec.value.ECFcode)
                    ecfrec.put_record(results, filespec.ECFCLUB_FILE_DEF)
                elif record[0] != utf8ify(ecfimp.value.CODE):
                    ecfrec.key.recno = 0
                    ecfrec.value.ECFcode = strify(ecfimp.value.CODE)
                    ecfrec.value.ECFname = strify(ecfimp.value.CLUB)
                    ecfrec.value.ECFcountycode = strify(ecfimp.value.COUNTY)
                    ecfrec.value.calculate_ecf_active_club()
                    ecf_codes.add(ecfrec.value.ECFcode)
                    ecfrec.put_record(results, filespec.ECFCLUB_FILE_DEF)
                else:
                    ecfrec.load_instance(
                        results,
                        filespec.ECFCLUB_FILE_DEF,
                        filespec.ECFCLUBCODE_FIELD_DEF,
                        record)
                    ecfnew = ecfrec.clone()
                    ecfnew.value.ECFname = strify(ecfimp.value.CLUB)
                    ecfnew.value.ECFcountycode = strify(ecfimp.value.COUNTY)
                    ecfnew.value.calculate_ecf_active_club()
                    ecf_codes.add(ecfrec.value.ECFcode)
                    ecfrec.edit_record(
                        results,
                        filespec.ECFCLUB_FILE_DEF,
                        filespec.ECFCLUBCODE_FIELD_DEF,
                        ecfnew)
                data = clubs.next()
        finally:
            clubs.close()

        # Do deletes implied by absence of ECF data
        record = ecfcursor.first()
        while record:
            ecfrec.load_instance(
                results,
                filespec.ECFCLUB_FILE_DEF,
                filespec.ECFCLUBCODE_FIELD_DEF,
                record)
            if ecfrec.value.ECFcode not in ecf_codes:
                ecfnew = ecfrec.clone()
                ecfnew.value.calculate_ecf_active_club()
                ecfrec.edit_record(
                    results,
                    filespec.ECFCLUB_FILE_DEF,
                    filespec.ECFCLUBCODE_FIELD_DEF,
                    ecfnew)
            record = ecfcursor.next()
    finally:
        ecfcursor.close()

    results.commit()
    if logwidget:
        logwidget.append_text('', timestamp=False)
        logwidget.append_text(' '.join(
            ['Master club file for',
             datecontrol,
             'has been processed.']))
    return True


def copy_ecf_players_post_2011_rules(
    results,
    logwidget=None,
    ecffile=None,
    ecfdate=None,
    parent=None,
    datecontrol=None,
    strify=_strify,
    utf8ify=_utf8ify,
    coerce_to_utf8=_coerce_to_utf8,
    **kwargs
    ):
    """Import a new ECF player file

    widget - the manager object for the ecf data import tab

    """
    if logwidget:
        logwidget.append_text('', timestamp=False)
        logwidget.append_text(''.join(
            ['Start processing Master player file for ',
             datecontrol,
             '.']))
    results.start_transaction()

    # Update Master file date record
    datecursor = results.database_cursor(
        filespec.ECFTXN_FILE_DEF, filespec.ECFDATE_FIELD_DEF)
    try:
        r = datecursor.first()
        ecfdateexists = False
        while r:
            daterecord = ecfrecord.ECFrefDBrecordECFdate()
            daterecord.load_instance(
                results,
                filespec.ECFTXN_FILE_DEF,
                filespec.ECFDATE_FIELD_DEF,
                r)
            if daterecord.value.ECFobjtype == ecfrecord.objtypePlayer:
                if ecfdate != daterecord.value.appliedECFdate:
                    newdaterecord = daterecord.clone()
                    newdaterecord.value.appliedECFdate = ecfdate
                    newdaterecord.edit_record(
                        results,
                        filespec.ECFTXN_FILE_DEF,
                        filespec.ECFDATE_FIELD_DEF,
                        newdaterecord)
                if ecfdate == daterecord.value.ECFdate:
                    ecfdateexists = True
            r = datecursor.next()
        if not ecfdateexists:
            txndaterec = ecfrecord.ECFrefDBrecordECFdate()
            txndaterec.value.ECFdate = ecfdate
            txndaterec.value.ECFtxntype = ecfrecord.txnNew
            txndaterec.value.ECFobjtype = ecfrecord.objtypePlayer
            txndaterec.value.appliedECFdate = ecfdate
            txndaterec.key.recno = 0
            txndaterec.put_record(results, filespec.ECFTXN_FILE_DEF)
            ecfrecord.ECFrefDBrecordECFdate.set_most_recent_master_dates(
                results)
    finally:
        datecursor.close()

    # Load the ECF data
    ecfimp = ecfplayerdb.ECFplayersDBrecord()
    if logwidget:
        logwidget.append_text('', timestamp=False)
        logwidget.append_text(
            'Add or edit ECF Grading Code references to Master player file.')
    ecf_codes = set()
    ecfcursor = results.database_cursor(
        filespec.ECFPLAYER_FILE_DEF, filespec.ECFPLAYERCODE_FIELD_DEF)
    try:
        players = ecffile.database_cursor(ecfplayerdb.PLAYERS,
                                          ecfplayerdb.PLAYERS)
        try:
            data = players.first()
            while data:
                ecfimp.load_instance(
                    ecffile, ecfplayerdb.PLAYERS, ecfplayerdb.PLAYERS, data)
                clubcodes = []
                for f in ecfrecord._ECFplayerclubsfields:
                    c = ecfimp.value.__dict__.get(f)
                    if c:
                        clubcodes.append(strify(c))
                clubcodes.sort()
                ecfrec = ecfrecord.ECFrefDBrecordECFplayer()
                record = ecfcursor.nearest(utf8ify(ecfimp.value.REF))
                if record == None:
                    ecfrec.key.recno = 0
                    ecfrec.value.ECFcode = strify(ecfimp.value.REF)
                    ecfrec.value.ECFactive = True
                    ecfrec.value.ECFname = strify(ecfimp.value.NAME)
                    ecfrec.value.ECFclubcodes = clubcodes
                    ecf_codes.add(ecfrec.value.ECFcode)
                    ecfrec.put_record(results, filespec.ECFPLAYER_FILE_DEF)
                elif record[0] != strify(ecfimp.value.REF):
                    ecfrec.key.recno = 0
                    ecfrec.value.ECFcode = strify(ecfimp.value.REF)
                    ecfrec.value.ECFactive = True
                    ecfrec.value.ECFname = strify(ecfimp.value.NAME)
                    ecfrec.value.ECFclubcodes = clubcodes
                    ecf_codes.add(ecfrec.value.ECFcode)
                    ecfrec.put_record(results, filespec.ECFPLAYER_FILE_DEF)
                else:
                    ecfrec.load_instance(
                        results,
                        filespec.ECFPLAYER_FILE_DEF,
                        filespec.ECFPLAYERCODE_FIELD_DEF,
                        record)
                    ecfnew = ecfrec.clone()
                    ecfnew.value.ECFactive = True
                    ecfnew.value.ECFname = strify(ecfimp.value.NAME)
                    ecfnew.value.ECFclubcodes = clubcodes
                    ecf_codes.add(ecfrec.value.ECFcode)
                    ecfrec.edit_record(
                        results,
                        filespec.ECFPLAYER_FILE_DEF,
                        filespec.ECFPLAYERCODE_FIELD_DEF,
                        ecfnew)
                data = players.next()
        finally:
            players.close()

        # Do deletes implied by absence of ECF data
        if logwidget:
            logwidget.append_text(
                'Delete ECF Grading Code references not on Master player file.')
        record = ecfcursor.first()
        while record:
            ecfrec = ecfrecord.ECFrefDBrecordECFplayer()
            ecfrec.load_instance(
                results,
                filespec.ECFPLAYER_FILE_DEF,
                filespec.ECFPLAYERCODE_FIELD_DEF,
                record)
            if ecfrec.value.ECFcode not in ecf_codes:
                ecfnew = ecfrec.clone()
                ecfnew.value.ECFactive = True
                ecfrec.edit_record(
                    results,
                    filespec.ECFPLAYER_FILE_DEF,
                    filespec.ECFPLAYERCODE_FIELD_DEF,
                    ecfnew)
            record = ecfcursor.next()
    finally:
        ecfcursor.close()

    # Match grading codes for new players to copied master list
    # Any left unlinked are probably merged before master list published
    # if publication after results submission
    if logwidget:
        logwidget.append_text(
            'Reconcile new player Grading Codes with Master player file.')
    ecfmapcursor = results.database_cursor(
        filespec.MAPECFPLAYER_FILE_DEF, filespec.MAPECFPLAYER_FIELD_DEF)
    try:
        mapdata = ecfmapcursor.first()
        while mapdata:
            mr = ecfmaprecord.ECFmapDBrecordPlayer()
            mr.load_record(mapdata)

            # mapdata values like (key, None) occur sometimes, origin unknown
            # but seen only when mixing event imports and ecf reference data
            # imports.
            # Ignoring them should be correct, and seems ok too.
            # Find and delete them offline.
            # See gui.events_lite too.
            if mr.value.__dict__:
                
                if mr.value.playercode is None:
                    if mr.value.playerecfcode is not None:
                        if ecfrecord.get_ecf_player_for_grading_code(
                            results, mr.value.playerecfcode):
                            newmr = mr.clone()
                            newmr.value.playerecfcode = None
                            newmr.value.playercode = mr.value.playerecfcode
                            mr.edit_record(
                                results,
                                filespec.MAPECFPLAYER_FILE_DEF,
                                filespec.MAPECFPLAYER_FIELD_DEF,
                                newmr)
            mapdata = ecfmapcursor.next()
    finally:
        ecfmapcursor.close()

    results.commit()
    if logwidget:
        logwidget.append_text('', timestamp=False)
        logwidget.append_text(''.join(
            ['Master player file for ',
             datecontrol,
             ' has been processed.']))
    return True


def copy_ecf_players_post_1998_rules(
    results,
    logwidget=None,
    ecffile=None,
    ecfdate=None,
    parent=None,
    datecontrol=None,
    strify=_strify,
    utf8ify=_utf8ify,
    coerce_to_utf8=_coerce_to_utf8,
    **kwargs
    ):
    """Import a new ECF player file

    widget - the manager object for the ecf data import tab

    """
    txndaterec = ecfrecord.ECFrefDBrecordECFdate()
    txndaterec.value.ECFdate = ecfdate
    txndaterec.value.ECFtxntype = ecfrecord.txnNew
    txndaterec.value.ECFobjtype = ecfrecord.objtypePlayer
    txndaterec.key.recno = 0
    daterecord = ecfrecord.ECFrefDBrecordECFdate()
    datecursor = results.database_cursor(
        filespec.ECFTXN_FILE_DEF, filespec.ECFDATE_FIELD_DEF)
    try:
        r = datecursor.nearest(coerce_to_utf8(ecfdate))
        while r:
            daterecord.load_instance(
                results,
                filespec.ECFTXN_FILE_DEF,
                filespec.ECFDATE_FIELD_DEF,
                r)
            if ecfdate != daterecord.value.ECFdate:
                break
            if txndaterec.value == daterecord.value:
                if logwidget:
                    logwidget.append_text('', timestamp=False)
                    logwidget.append_text(' '.join(
                        ['A Master player file for',
                         datecontrol,
                         'has been input previously.']))
                return False
            r = datecursor.next()
    finally:
        datecursor.close()
    results.start_transaction()
    txndaterec.put_record(results, filespec.ECFTXN_FILE_DEF)
    ecfrecord.ECFrefDBrecordECFdate.set_most_recent_master_dates(results)

    ecfrec = ecfrecord.ECFrefDBrecordECFplayer()
    ecfimp = ecfplayerdb.ECFplayersDBrecord()

    ecfcursor = results.database_cursor(
        filespec.ECFPLAYER_FILE_DEF, filespec.ECFPLAYERCODE_FIELD_DEF)
    try:
        players = ecffile.database_cursor(ecfplayerdb.PLAYERS,
                                          ecfplayerdb.PLAYERS)
        
        # Load the ECF data
        ecf_codes = set()
        try:
            data = players.first()
            while data:
                ecfimp.load_instance(
                    ecffile, ecfplayerdb.PLAYERS, ecfplayerdb.PLAYERS, data)
                clubcodes = []
                for f in ecfrecord._ECFplayerclubsfields:
                    c = ecfimp.value.__dict__.get(f)
                    if c:
                        clubcodes.append(strify(c))
                record = ecfcursor.nearest(utf8ify(ecfimp.value.REF))
                if record == None:
                    ecfrec.key.recno = 0
                    ecfrec.value.ECFcode = strify(ecfimp.value.REF)
                    ecfrec.value.ECFname = strify(ecfimp.value.NAME)
                    ecfrec.value.ECFclubcodes = clubcodes
                    ecfrec.value.calculate_ecf_active_player()
                    ecf_codes.add(ecfrec.value.ECFcode)
                    ecfrec.put_record(results, filespec.ECFPLAYER_FILE_DEF)
                elif record[0] != utf8ify(ecfimp.value.REF):
                    ecfrec.key.recno = 0
                    ecfrec.value.ECFcode = strify(ecfimp.value.REF)
                    ecfrec.value.ECFname = strify(ecfimp.value.NAME)
                    ecfrec.value.ECFclubcodes = clubcodes
                    ecfrec.value.calculate_ecf_active_player()
                    ecf_codes.add(ecfrec.value.ECFcode)
                    ecfrec.put_record(results, filespec.ECFPLAYER_FILE_DEF)
                else:
                    ecfrec.load_instance(
                        results,
                        filespec.ECFPLAYER_FILE_DEF,
                        filespec.ECFPLAYERCODE_FIELD_DEF,
                        record)
                    ecfnew = ecfrec.clone()
                    ecfnew.value.ECFname = strify(ecfimp.value.NAME)
                    ecfnew.value.ECFclubcodes = clubcodes
                    ecfnew.value.calculate_ecf_active_player()
                    ecf_codes.add(ecfrec.value.ECFcode)
                    ecfrec.edit_record(
                        results,
                        filespec.ECFPLAYER_FILE_DEF,
                        filespec.ECFPLAYERCODE_FIELD_DEF,
                        ecfnew)
                data = players.next()
        finally:
            players.close()

        # Do deletes implied by absence of ECF data
        record = ecfcursor.first()
        while record:
            ecfrec.load_instance(
                results,
                filespec.ECFPLAYER_FILE_DEF,
                filespec.ECFPLAYERCODE_FIELD_DEF,
                record)
            if ecfrec.value.ECFcode not in ecf_codes:
                ecfnew = ecfrec.clone()
                ecfnew.value.calculate_ecf_active_player()
                ecfrec.edit_record(
                    results,
                    filespec.ECFPLAYER_FILE_DEF,
                    filespec.ECFPLAYERCODE_FIELD_DEF,
                    ecfnew)
            record = ecfcursor.next()
    finally:
        ecfcursor.close()

    # Match grading codes for new players to copied master list
    # Any left unlinked are probably merged before master list published
    # if publication after results submission
    mr = ecfmaprecord.ECFmapDBrecordPlayer()
    ecfmapcursor = results.database_cursor(
        filespec.MAPECFPLAYER_FILE_DEF, filespec.MAPECFPLAYER_FIELD_DEF)
    try:
        mapdata = ecfmapcursor.first()
        while mapdata:
            mr.load_record(mapdata)
            if mr.value.playercode is None:
                if mr.value.playerecfcode is not None:
                    if ecfrecord.get_ecf_player_for_grading_code(
                        results, mr.value.playerecfcode):
                        newmr = mr.clone()
                        newmr.value.playerecfcode = None
                        newmr.value.playercode = mr.value.playerecfcode
                        mr.edit_record(
                            results,
                            filespec.MAPECFPLAYER_FILE_DEF,
                            filespec.MAPECFPLAYER_FIELD_DEF,
                            newmr)
            mapdata = ecfmapcursor.next()
    finally:
        ecfmapcursor.close()

    results.commit()
    if logwidget:
        logwidget.append_text('', timestamp=False)
        logwidget.append_text(' '.join(
            ['Master player file for',
             datecontrol,
             'has been processed.']))
    return True


def copy_ecf_clubs_post_2011_rules(
    results,
    logwidget=None,
    ecffile=None,
    ecfdate=None,
    parent=None,
    datecontrol=None,
    strify=_strify,
    utf8ify=_utf8ify,
    coerce_to_utf8=_coerce_to_utf8,
    keyconvert=_strify,
    datekey=lambda d: d,
    **kwargs
    ):
    """Import a new ECF club file

    widget - the manager object for the ecf data import tab

    """
    if logwidget:
        logwidget.append_text('', timestamp=False)
        logwidget.append_text(''.join(
            ['Start processing Master club file for ',
             datecontrol,
             '.']))
    results.start_transaction()

    # Update Master file date record
    datecursor = results.database_cursor(
        filespec.ECFTXN_FILE_DEF, filespec.ECFDATE_FIELD_DEF)
    try:
        r = datecursor.first()
        ecfdateexists = False
        while r:
            daterecord = ecfrecord.ECFrefDBrecordECFdate()
            daterecord.load_instance(
                results,
                filespec.ECFTXN_FILE_DEF,
                filespec.ECFDATE_FIELD_DEF,
                r)
            if daterecord.value.ECFobjtype == ecfrecord.objtypeClub:
                if ecfdate != daterecord.value.appliedECFdate:
                    newdaterecord = daterecord.clone()
                    newdaterecord.value.appliedECFdate = ecfdate
                    newdaterecord.edit_record(
                        results,
                        filespec.ECFTXN_FILE_DEF,
                        filespec.ECFDATE_FIELD_DEF,
                        newdaterecord)
                if ecfdate == daterecord.value.ECFdate:
                    ecfdateexists = True
            r = datecursor.next()
        if not ecfdateexists:
            txndaterec = ecfrecord.ECFrefDBrecordECFdate()
            txndaterec.value.ECFdate = ecfdate
            txndaterec.value.ECFtxntype = ecfrecord.txnNew
            txndaterec.value.ECFobjtype = ecfrecord.objtypeClub
            txndaterec.value.appliedECFdate = ecfdate
            txndaterec.key.recno = 0
            txndaterec.put_record(results, filespec.ECFTXN_FILE_DEF)
            ecfrecord.ECFrefDBrecordECFdate.set_most_recent_master_dates(
                results)
    finally:
        datecursor.close()

    # Load the ECF data
    ecfimp = ecfclubdb.ECFclubsDBrecord()
    if logwidget:
        logwidget.append_text('', timestamp=False)
        logwidget.append_text(
            'Add or edit ECF Club Code references to Master club file.')
    ecf_codes = set()
    ecfcursor = results.database_cursor(
        filespec.ECFCLUB_FILE_DEF, filespec.ECFCLUBCODE_FIELD_DEF)
    try:
        clubs = ecffile.database_cursor(ecfclubdb.CLUBS, ecfclubdb.CLUBS)
        try:
            data = clubs.first()
            while data:
                ecfimp.load_instance(
                    ecffile, ecfclubdb.CLUBS, ecfclubdb.CLUBS, data)
                ecfrec = ecfrecord.ECFrefDBrecordECFclub()
                record = ecfcursor.nearest(keyconvert(ecfimp.value.CODE))
                if record == None:
                    ecfrec.key.recno = 0
                    ecfrec.value.ECFcode = strify(ecfimp.value.CODE)
                    ecfrec.value.ECFactive = True
                    ecfrec.value.ECFname = strify(ecfimp.value.CLUB)
                    ecfrec.value.ECFcountycode = strify(ecfimp.value.COUNTY)
                    ecf_codes.add(ecfrec.value.ECFcode)
                    ecfrec.put_record(results, filespec.ECFCLUB_FILE_DEF)
                elif record[0] != strify(ecfimp.value.CODE):
                    ecfrec.key.recno = 0
                    ecfrec.value.ECFcode = strify(ecfimp.value.CODE)
                    ecfrec.value.ECFactive = True
                    ecfrec.value.ECFname = strify(ecfimp.value.CLUB)
                    ecfrec.value.ECFcountycode = strify(ecfimp.value.COUNTY)
                    ecf_codes.add(ecfrec.value.ECFcode)
                    ecfrec.put_record(results, filespec.ECFCLUB_FILE_DEF)
                else:
                    ecfrec.load_instance(
                        results,
                        filespec.ECFCLUB_FILE_DEF,
                        filespec.ECFCLUBCODE_FIELD_DEF,
                        record)
                    ecfnew = ecfrec.clone()
                    ecfnew.value.ECFactive = True
                    ecfnew.value.ECFname = strify(ecfimp.value.CLUB)
                    ecfnew.value.ECFcountycode = strify(ecfimp.value.COUNTY)
                    ecf_codes.add(ecfrec.value.ECFcode)
                    ecfrec.edit_record(
                        results,
                        filespec.ECFCLUB_FILE_DEF,
                        filespec.ECFCLUBCODE_FIELD_DEF,
                        ecfnew)
                data = clubs.next()
        finally:
            clubs.close()

        # Do deletes implied by absence of ECF data
        if logwidget:
            logwidget.append_text(
                'Delete ECF Club Code references not on Master club file.')
        record = ecfcursor.first()
        while record:
            ecfrec = ecfrecord.ECFrefDBrecordECFclub()
            ecfrec.load_instance(
                results,
                filespec.ECFCLUB_FILE_DEF,
                filespec.ECFCLUBCODE_FIELD_DEF,
                record)
            if ecfrec.value.ECFcode not in ecf_codes:
                ecfnew = ecfrec.clone()
                ecfnew.value.ECFactive = None
                ecfrec.edit_record(
                    results,
                    filespec.ECFCLUB_FILE_DEF,
                    filespec.ECFCLUBCODE_FIELD_DEF,
                    ecfnew)
            record = ecfcursor.next()
    finally:
        ecfcursor.close()

    results.commit()
    if logwidget:
        logwidget.append_text('', timestamp=False)
        logwidget.append_text(' '.join(
            ['Master club file for',
             datecontrol,
             'has been processed.']))
    return True


def update_ecf_clubs_post_2006_rules(
    results,
    logwidget=None,
    ecffile=None,
    ecfdate=None,
    parent=None,
    datecontrol=None,
    strify=_strify,
    utf8ify=_utf8ify,
    coerce_to_utf8=_coerce_to_utf8,
    **kwargs
    ):
    """Import an ECF club file update

    widget - the manager object for the ecf data import tab

    """
    txndaterec = ecfrecord.ECFrefDBrecordECFdate()
    txndaterec.value.ECFdate = ecfdate
    txndaterec.value.ECFtxntype = ecfrecord.txnUpdate
    txndaterec.value.ECFobjtype = ecfrecord.objtypeClub
    txndaterec.key.recno = 0
    daterecord = ecfrecord.ECFrefDBrecordECFdate()
    datecursor = results.database_cursor(
        filespec.ECFTXN_FILE_DEF, filespec.ECFDATE_FIELD_DEF)
    try:
        r = datecursor.nearest(coerce_to_utf8(ecfdate))
        while r:
            daterecord.load_instance(
                results,
                filespec.ECFTXN_FILE_DEF,
                filespec.ECFDATE_FIELD_DEF,
                r)
            if ecfdate != daterecord.value.ECFdate:
                break
            if txndaterec.value == daterecord.value:
                if logwidget:
                    logwidget.append_text('', timestamp=False)
                    logwidget.append_text(' '.join(
                        ['A Master club file update for',
                         datecontrol,
                         'has been input previously.']))
                return False
            r = datecursor.next()
    finally:
        datecursor.close()
    results.start_transaction()
    txndaterec.put_record(results, filespec.ECFTXN_FILE_DEF)
    ecfrecord.ECFrefDBrecordECFdate.set_most_recent_master_dates(results)

    ecfrec = ecfrecord.ECFrefDBrecordECFclub()
    ecfimp = ecfclubdb.ECFclubsDBrecord()

    ecfcursor = results.database_cursor(
        filespec.ECFCLUB_FILE_DEF, filespec.ECFCLUBCODE_FIELD_DEF)
    clubs = ecffile.database_cursor(ecfclubdb.CLUBS, ecfclubdb.CLUBS)
    
    # Load the ECF data
    try:
        data = clubs.first()
        while data:
            ecfimp.load_instance(
                ecffile, ecfclubdb.CLUBS, ecfclubdb.CLUBS, data)
            record = ecfcursor.nearest(utf8ify(ecfimp.value.CODE))
            txntype = ecfrecord._ecfupdateclubtxns.get(
                ecfimp.value.RECTYPE, ecfrecord._badtxn)
            if record == None:
                ecfrec.key.recno = 0
                ecfrec.value.ECFcode = strify(ecfimp.value.CODE)
                ecfrec.value.ECFname = strify(ecfimp.value.CLUB)
                ecfrec.value.ECFcountycode = strify(ecfimp.value.COUNTY)
                ecfrec.value.calculate_ecf_active_club()
                ecfrec.put_record(results, filespec.ECFCLUB_FILE_DEF)
            elif record[0] != utf8ify(ecfimp.value.CODE):
                ecfrec.key.recno = 0
                ecfrec.value.ECFcode = strify(ecfimp.value.CODE)
                ecfrec.value.ECFname = strify(ecfimp.value.CLUB)
                ecfrec.value.ECFcountycode = strify(ecfimp.value.COUNTY)
                ecfrec.value.calculate_ecf_active_club()
                ecfrec.put_record(results, filespec.ECFCLUB_FILE_DEF)
            else:
                ecfrec.load_instance(
                    results,
                    filespec.ECFCLUB_FILE_DEF,
                    filespec.ECFCLUBCODE_FIELD_DEF,
                    record)
                ecfnew = ecfrec.clone()
                ecfnew.value.ECFname = strify(ecfimp.value.CLUB)
                ecfnew.value.ECFcountycode = strify(ecfimp.value.COUNTY)
                ecfnew.value.calculate_ecf_active_club()
                ecfrec.edit_record(
                    results,
                    filespec.ECFCLUB_FILE_DEF,
                    filespec.ECFCLUBCODE_FIELD_DEF,
                    ecfnew)
            data = clubs.next()
    finally:
        ecfcursor.close()
        clubs.close()

    results.commit()
    if logwidget:
        logwidget.append_text('', timestamp=False)
        logwidget.append_text(' '.join(
            ['Master club file update for',
             datecontrol,
             'has been processed.']))
    return True


def update_ecf_players_post_2006_rules(
    results,
    logwidget=None,
    ecffile=None,
    ecfdate=None,
    parent=None,
    datecontrol=None,
    strify=_strify,
    utf8ify=_utf8ify,
    coerce_to_utf8=_coerce_to_utf8,
    **kwargs
    ):
    """Import an ECF player file update

    widget - the manager object for the ecf data import tab

    """
    txndaterec = ecfrecord.ECFrefDBrecordECFdate()
    txndaterec.value.ECFdate = ecfdate
    txndaterec.value.ECFtxntype = ecfrecord.txnUpdate
    txndaterec.value.ECFobjtype = ecfrecord.objtypePlayer
    txndaterec.key.recno = 0
    daterecord = ecfrecord.ECFrefDBrecordECFdate()
    datecursor = results.database_cursor(
        filespec.ECFTXN_FILE_DEF, filespec.ECFDATE_FIELD_DEF)
    try:
        r = datecursor.nearest(coerce_to_utf8(ecfdate))
        while r:
            daterecord.load_instance(
                results,
                filespec.ECFTXN_FILE_DEF,
                filespec.ECFDATE_FIELD_DEF,
                r)
            if ecfdate != daterecord.value.ECFdate:
                break
            if txndaterec.value == daterecord.value:
                if logwidget:
                    logwidget.append_text('', timestamp=False)
                    logwidget.append_text(' '.join(
                        ['A Master player file update for',
                         datecontrol,
                         'has been input previously.']))
                return False
            r = datecursor.next()
    finally:
        datecursor.close()
    results.start_transaction()
    txndaterec.put_record(results, filespec.ECFTXN_FILE_DEF)
    ecfrecord.ECFrefDBrecordECFdate.set_most_recent_master_dates(results)

    ecfrec = ecfrecord.ECFrefDBrecordECFplayer()
    ecfimp = ecfplayerdb.ECFplayersDBrecord()

    ecfcursor = results.database_cursor(
        filespec.ECFPLAYER_FILE_DEF, filespec.ECFPLAYERCODE_FIELD_DEF)
    players = ecffile.database_cursor(ecfplayerdb.PLAYERS, ecfplayerdb.PLAYERS)
    
    # Load the ECF data
    try:
        data = players.first()
        while data:
            ecfimp.load_instance(
                ecffile, ecfplayerdb.PLAYERS, ecfplayerdb.PLAYERS, data)
            clubcodes = []
            for f in ecfrecord._ECFplayerclubsfields:
                c = ecfimp.value.__dict__.get(f)
                if c:
                    clubcodes.append(strify(c))
            record = ecfcursor.nearest(utf8ify(ecfimp.value.REF))
            # ECFactive to depend on txntype
            txntype = ecfrecord._ecfupdateplayertxns.get(
                ecfimp.value.RECTYPE, ecfrecord._badtxn)
            if record == None:
                ecfrec.key.recno = 0
                ecfrec.value.ECFcode = strify(ecfimp.value.REF)
                ecfrec.value.ECFname = strify(ecfimp.value.NAME)
                ecfrec.value.ECFclubcodes = clubcodes
                ecfrec.value.calculate_ecf_active_player()
                ecfrec.put_record(results, filespec.ECFPLAYER_FILE_DEF)
            elif record[0] != utf8ify(ecfimp.value.REF):
                ecfrec.key.recno = 0
                ecfrec.value.ECFcode = strify(ecfimp.value.REF)
                ecfrec.value.ECFname = strify(ecfimp.value.NAME)
                ecfrec.value.ECFclubcodes = clubcodes
                ecfrec.value.calculate_ecf_active_player()
                ecfrec.put_record(results, filespec.ECFPLAYER_FILE_DEF)
            else:
                ecfrec.load_instance(
                    results,
                    filespec.ECFPLAYER_FILE_DEF,
                    filespec.ECFPLAYERCODE_FIELD_DEF,
                    record)
                ecfnew = ecfrec.clone()
                ecfnew.value.ECFname = strify(ecfimp.value.NAME)
                ecfnew.value.ECFclubcodes = clubcodes
                ecfnew.value.calculate_ecf_active_player()
                ecfrec.edit_record(
                    results,
                    filespec.ECFPLAYER_FILE_DEF,
                    filespec.ECFPLAYERCODE_FIELD_DEF,
                    ecfnew)
            data = players.next()
    finally:
        ecfcursor.close()
        players.close()

    results.commit()
    if logwidget:
        logwidget.append_text('', timestamp=False)
        logwidget.append_text(' '.join(
            ['Master player file update for',
             datecontrol,
             'has been processed.']))
    return True


def update_ecf_clubs_post_2011_rules(
    results,
    logwidget=None,
    ecffile=None,
    ecfdate=None,
    parent=None,
    datecontrol=None,
    strify=_strify,
    utf8ify=_utf8ify,
    coerce_to_utf8=_coerce_to_utf8,
    **kwargs
    ):
    """Import an ECF club file update

    widget - the manager object for the ecf data import tab

    """
    txndaterec = ecfrecord.ECFrefDBrecordECFdate()
    txndaterec.value.ECFdate = ecfdate
    txndaterec.value.ECFtxntype = ecfrecord.txnUpdate
    txndaterec.value.ECFobjtype = ecfrecord.objtypeClub
    txndaterec.key.recno = 0
    daterecord = ecfrecord.ECFrefDBrecordECFdate()
    datecursor = results.database_cursor(
        filespec.ECFTXN_FILE_DEF, filespec.ECFDATE_FIELD_DEF)
    try:
        r = datecursor.nearest(coerce_to_utf8(ecfdate))
        while r:
            daterecord.load_instance(
                results,
                filespec.ECFTXN_FILE_DEF,
                filespec.ECFDATE_FIELD_DEF,
                r)
            if ecfdate != daterecord.value.ECFdate:
                break
            if txndaterec.value == daterecord.value:
                if logwidget:
                    logwidget.append_text('', timestamp=False)
                    logwidget.append_text(' '.join(
                        ['A Master club file update for',
                         datecontrol,
                         'has been input previously.']))
                return False
            r = datecursor.next()
    finally:
        datecursor.close()
    results.start_transaction()
    txndaterec.put_record(results, filespec.ECFTXN_FILE_DEF)
    ecfrecord.ECFrefDBrecordECFdate.set_most_recent_master_dates(results)

    ecfrec = ecfrecord.ECFrefDBrecordECFclub()
    ecfimp = ecfclubdb.ECFclubsDBrecord()

    ecfcursor = results.database_cursor(
        filespec.ECFCLUB_FILE_DEF, filespec.ECFCLUBCODE_FIELD_DEF)
    clubs = ecffile.database_cursor(ecfclubdb.CLUBS, ecfclubdb.CLUBS)

    # Load the ECF data
    try:
        data = clubs.first()
        while data:
            ecfimp.load_instance(
                ecffile, ecfclubdb.CLUBS, ecfclubdb.CLUBS, data)
            record = ecfcursor.nearest(utf8ify(ecfimp.value.CODE))
            # ECFactive to depend on txntype
            txntype = ecfrecord._ecfupdateclubtxns.get(
                ecfimp.value.RECTYPE, ecfrecord._badtxn)
            if record == None:
                ecfrec.key.recno = 0
                ecfrec.value.ECFcode = strify(ecfimp.value.CODE)
                ecfnew.value.ECFactive = True
                ecfrec.value.ECFname = strify(ecfimp.value.CLUB)
                ecfrec.value.ECFcountycode = strify(ecfimp.value.COUNTY)
                ecfrec.put_record(results, filespec.ECFCLUB_FILE_DEF)
            elif record[0] != utf8ify(ecfimp.value.CODE):
                ecfrec.key.recno = 0
                ecfrec.value.ECFcode = strify(ecfimp.value.CODE)
                ecfnew.value.ECFactive = True
                ecfrec.value.ECFname = strify(ecfimp.value.CLUB)
                ecfrec.value.ECFcountycode = strify(ecfimp.value.COUNTY)
                ecfrec.put_record(results, filespec.ECFCLUB_FILE_DEF)
            else:
                ecfrec.load_instance(
                    results,
                    filespec.ECFCLUB_FILE_DEF,
                    filespec.ECFCLUBCODE_FIELD_DEF,
                    record)
                ecfnew = ecfrec.clone()
                ecfnew.value.ECFactive = True
                ecfnew.value.ECFname = strify(ecfimp.value.CLUB)
                ecfnew.value.ECFcountycode = strify(ecfimp.value.COUNTY)
                ecfrec.edit_record(
                    results,
                    filespec.ECFCLUB_FILE_DEF,
                    filespec.ECFCLUBCODE_FIELD_DEF,
                    ecfnew)
            data = clubs.next()
    finally:
        ecfcursor.close()
        clubs.close()

    results.commit()
    if logwidget:
        logwidget.append_text('', timestamp=False)
        logwidget.append_text(' '.join(
            ['Master club file update for',
             datecontrol,
             'has been processed.']))
    return True


def update_ecf_players_post_2011_rules(
    results,
    logwidget=None,
    ecffile=None,
    ecfdate=None,
    parent=None,
    datecontrol=None,
    strify=_strify,
    utf8ify=_utf8ify,
    coerce_to_utf8=_coerce_to_utf8,
    **kwargs
    ):
    """Import an ECF player file update

    widget - the manager object for the ecf data import tab

    """
    txndaterec = ecfrecord.ECFrefDBrecordECFdate()
    txndaterec.value.ECFdate = ecfdate
    txndaterec.value.ECFtxntype = ecfrecord.txnUpdate
    txndaterec.value.ECFobjtype = ecfrecord.objtypePlayer
    txndaterec.key.recno = 0
    daterecord = ecfrecord.ECFrefDBrecordECFdate()
    datecursor = results.database_cursor(
        filespec.ECFTXN_FILE_DEF, filespec.ECFDATE_FIELD_DEF)
    try:
        r = datecursor.nearest(coerce_to_utf8(ecfdate))
        while r:
            daterecord.load_instance(
                results,
                filespec.ECFTXN_FILE_DEF,
                filespec.ECFDATE_FIELD_DEF,
                r)
            if ecfdate != daterecord.value.ECFdate:
                break
            if txndaterec.value == daterecord.value:
                if logwidget:
                    logwidget.append_text('', timestamp=False)
                    logwidget.append_text(' '.join(
                        ['A Master player file update for',
                         datecontrol,
                         'has been input previously.']))
                return False
            r = datecursor.next()
    finally:
        datecursor.close()
    results.start_transaction()
    txndaterec.put_record(results, filespec.ECFTXN_FILE_DEF)
    ecfrecord.ECFrefDBrecordECFdate.set_most_recent_master_dates(results)

    ecfrec = ecfrecord.ECFrefDBrecordECFplayer()
    ecfimp = ecfplayerdb.ECFplayersDBrecord()

    ecfcursor = results.database_cursor(
        filespec.ECFPLAYER_FILE_DEF, filespec.ECFPLAYERCODE_FIELD_DEF)
    players = ecffile.database_cursor(ecfplayerdb.PLAYERS,
                                      ecfplayerdb.PLAYERS)

    # Load the ECF data
    try:
        data = players.first()
        while data:
            ecfimp.load_instance(
                ecffile, ecfplayerdb.PLAYERS, ecfplayerdb.PLAYERS, data)
            clubcodes = []
            for f in ecfrecord._ECFplayerclubsfields:
                c = ecfimp.value.__dict__.get(f)
                if c:
                    clubcodes.append(strify(c))
            record = ecfcursor.nearest(utf8ify(ecfimp.value.REF))
            # ECFactive to depend on txntype
            txntype = ecfrecord._ecfupdateplayertxns.get(
                ecfimp.value.RECTYPE, ecfrecord._badtxn)
            if record == None:
                ecfrec.key.recno = 0
                ecfrec.value.ECFcode = strify(ecfimp.value.REF)
                ecfrec.value.ECFactive = True
                ecfrec.value.ECFname = strify(ecfimp.value.NAME)
                ecfrec.value.ECFclubcodes = clubcodes
                ecfrec.put_record(results, filespec.ECFPLAYER_FILE_DEF)
            elif record[0] != utf8ify(ecfimp.value.REF):
                ecfrec.key.recno = 0
                ecfrec.value.ECFcode = strify(ecfimp.value.REF)
                ecfrec.value.ECFactive = True
                ecfrec.value.ECFname = strify(ecfimp.value.NAME)
                ecfrec.value.ECFclubcodes = clubcodes
                ecfrec.put_record(results, filespec.ECFPLAYER_FILE_DEF)
            else:
                ecfrec.load_instance(
                    results,
                    filespec.ECFPLAYER_FILE_DEF,
                    filespec.ECFPLAYERCODE_FIELD_DEF,
                    record)
                ecfnew = ecfrec.clone()
                ecfnew.value.ECFactive = True
                ecfnew.value.ECFname = strify(ecfimp.value.NAME)
                ecfrec.value.ECFclubcodes = clubcodes
                ecfrec.edit_record(
                    results,
                    filespec.ECFPLAYER_FILE_DEF,
                    filespec.ECFPLAYERCODE_FIELD_DEF,
                    ecfnew)
            data = players.next()
    finally:
        ecfcursor.close()
        players.close()

    results.commit()
    if logwidget:
        logwidget.append_text('', timestamp=False)
        logwidget.append_text(' '.join(
            ['Master player file update for',
             datecontrol,
             'has been processed.']))
    return True
