# resultsdatabase.py
# Copyright 2009 Roger Marsh
# Licence: See LICENCE (BSD licence)

"""Results database using DPT database via dptdb.dptapi.
"""

import os
import shutil

from basesup.dptapi import DPTapi

from rmappsup.workarounds import dialogues

from ..core.filespec import FileSpec
from .. import APPLICATION_NAME, ERROR_LOG


class ResultsDatabase(DPTapi):
    """Provide access to a database of results of games of chess."""

    _datasourceset_modulename = 'gridsup.dpt.dptdatasourceset'
    _knownnames_modulename = 'results.dpt.knownnames'

    def __init__(self, databasefolder, **kargs):

        try:
            sysprint = kargs.pop('sysprint')
        except KeyError:
            sysprint = 'CONSOLE'
        ddnames = FileSpec(**kargs)

        super(ResultsDatabase, self).__init__(
            ddnames,
            databasefolder,
            sysprint=sysprint,
            **kargs)

    def delete_database(self):
        """Close and delete the open results database."""
        names = set()
        basenames = set()
        names.add(self.get_dptsysfolder())
        basenames.add(os.path.basename(self.get_dptsysfolder()))
        for fn in (ERROR_LOG,):
            defer = os.path.join(self.get_database_folder(), fn)
            if os.path.exists(defer):
                names.add(defer)
                basenames.add(os.path.basename(defer))
        for k, v in self.get_dbsets().items():
            names.add(v._file)
            basenames.add(os.path.basename(v._file))
        listnames = [n for n in os.listdir(self.get_database_folder())
                     if n not in basenames]
        if len(listnames):
            dialogues.showinfo(
                title='Delete',
                message=''.join(
                    ('There is at least one file or folder in\n\n',
                     self.get_database_folder(),
                     '\n\nwhich may not be part of the database.  These items ',
                     'will not be deleted by Results.',
                     ))
                )
        self.close_database()
        for n in names:
            if os.path.isdir(n):
                shutil.rmtree(n, ignore_errors=True)
            else:
                os.remove(n)
        try:
            os.rmdir(self.get_database_folder())
        except:
            pass
        return True

    def open_context(self):
        """Open all files if they are in Normal mode (FISTAT == 0)."""
        oc = super(ResultsDatabase, self).open_context()
        if not oc:
            dialogues.showinfo(
                title='Open',
                message=''.join(
                    (APPLICATION_NAME,
                     ' is unable to open the database.\n\nRestore the ',
                     'database from backups, or source data, before trying ',
                     'again.',
                     )),
                )
            return 'Unable to open database'
        fistat = dict()
        for dbo in self.get_dbsets().values():
            fistat[dbo] = dbo.get_file_parameters(self.get_dbserv())['FISTAT']
        for dbo in self.get_dbsets().values():
            if fistat[dbo][0] != 0:
                break
        else:
            self.increase_database_size(files=None)
            return oc
        # At least one file is not in Normal state
        r = '\n'.join(
            ['\t'.join(
                (os.path.basename(dbo._file),
                 fistat[dbo][1]))
             for dbo in self.get_dbsets().values()])
        dialogues.showinfo(
            title='Open',
            message=''.join(
                (APPLICATION_NAME,
                 ' has opened the database but some of the files are ',
                 'not in the Normal state.\n\n',
                 r,
                 '\n\n',
                 APPLICATION_NAME,
                 ' will close the database on dismissing this ',
                 'dialogue.\n\nRestore the database from backups, or source ',
                 'data, before trying again.',
                 )),
            )
        return self.close_database()
