# leagues.py
# Copyright 2008 Roger Marsh
#Licence: See LICENCE (BSD licence)

"""Results database Leagues frame class.
"""

import os
import importlib

from rmappsup.workarounds import dialogues

from . import control
from . import events
from . import newplayers
from . import players
from . import ecfgradingcodes
from . import ecfclubcodes
from . import ecfevents
from . import ecfplayers
from . import newevent
from . import leagues_lite
from . import importecfdata
from . import feedback
from . import joineventplayers
from .. import ECF_DATA_IMPORT_MODULE
from .. import KNOWN_NAME_DATASOURCE_MODULE


class Leagues(leagues_lite.Leagues):
    
    """The Results frame for a Results database.
    """
    
    _tab_ecfeventdetail = 15
    _tab_ecfgradingcodes = 31
    _tab_ecfclubcodes = 32
    _tab_ecfevents = 33
    _tab_ecfplayers = 38
    _tab_importecfdata = 39
    _tab_importfeedback = 40
    _tab_joineventplayers = 43

    _state_ecfeventdetail = 5
    _state_importecfdata = 9
    _state_importfeedback = 10
    _state_joineventplayers = 13

    def __init__(self, master=None, cnf=dict(), **kargs):
        """Extend and define the results database results frame."""
        super(Leagues, self).__init__(
            master=master,
            cnf=cnf,
            **kargs)
        
        self._show_master_list_grading_codes = True
        
        self.define_tab(
            self._tab_control,
            text='Administration',
            tooltip='Open and close databases and import data.',
            underline=0,
            tabclass=lambda **k: control.Control(**k),
            create_actions=(control.Control._btn_opendatabase,),
            destroy_actions=(control.Control._btn_closedatabase,))
        self.define_tab(
            self._tab_events,
            text='Events',
            tooltip=' '.join((
                'Export event data and initiate preparation of results',
                'for submission to ECF')),
            underline=0,
            tabclass=lambda **k: events.Events(gridhorizontal=False, **k),
            destroy_actions=(control.Control._btn_closedatabase,))
        self.define_tab(
            self._tab_newplayers,
            text='NewPlayers',
            tooltip='Identify new players and merge with existing players.',
            underline=0,
            tabclass=lambda **k: newplayers.NewPlayers(
                gridhorizontal=False, **k),
            destroy_actions=(control.Control._btn_closedatabase,))
        self.define_tab(
            self._tab_players,
            text='Players',
            tooltip='Merge or separate existing players.',
            underline=0,
            tabclass=lambda **k: players.Players(gridhorizontal=False, **k),
            destroy_actions=(control.Control._btn_closedatabase,))
        self.define_tab(
            self._tab_ecfgradingcodes,
            text='Grading Codes',
            tooltip='Associate player with ECF grading code.',
            underline=0,
            tabclass=lambda **k: ecfgradingcodes.ECFGradingCodes(
                gridhorizontal=False, **k),
            destroy_actions=(control.Control._btn_closedatabase,))
        self.define_tab(
            self._tab_ecfclubcodes,
            text='Club Codes',
            tooltip='Associate player with ECF club code.',
            underline=0,
            tabclass=lambda **k: ecfclubcodes.ECFClubCodes(
                gridhorizontal=False, **k),
            destroy_actions=(control.Control._btn_closedatabase,))
        self.define_tab(
            self._tab_ecfevents,
            text='ECF Events',
            tooltip='Select an event to submit to ECF.',
            underline=5,
            tabclass=lambda **k: ecfevents.ECFEvents(gridhorizontal=False, **k),
            destroy_actions=(control.Control._btn_closedatabase,))
        self.define_tab(
            self._tab_ecfeventdetail,
            text='ECF Event Detail',
            tooltip='Update details of event for submission to ECF.',
            tabclass=lambda **k: newevent.NewEvent(**k),
            destroy_actions=(newevent.NewEvent._btn_cancel,))
        self.define_tab(
            self._tab_ecfplayers,
            text='Player ECF Detail',
            tooltip='Grading codes and club codes allocated to players.',
            underline=3,
            tabclass=lambda **k: ecfplayers.ECFPlayers(
                gridhorizontal=False, **k),
            destroy_actions=(control.Control._btn_closedatabase,))
        self.define_tab(
            self._tab_importecfdata,
            text='Import ECF Reference Data',
            tooltip='Import data from ECF Master and Update zipped files.',
            underline=-1,
            tabclass=lambda **k: importecfdata.ImportECFData(**k),
            destroy_actions=(importecfdata.ImportECFData._btn_closeecfimport,))
        self.define_tab(
            self._tab_importfeedback,
            text='Feedback',
            tooltip='Import data from ECF feedback text files.',
            underline=-1,
            tabclass=lambda **k: feedback.Feedback(**k),
            destroy_actions=(feedback.Feedback._btn_closefeedback,))
        self.define_tab(
            self._tab_joineventplayers,
            text='Join Event Players',
            tooltip="Join an event's players with same-named players.",
            underline=-1,
            tabclass=lambda **k: joineventplayers.JoinEventPlayers(**k),
            destroy_actions=(joineventplayers.JoinEventPlayers._btn_cancel,))

        self.define_state_transitions(
            tab_state={
                self._state_dbopen: (
                    self._tab_control,
                    self._tab_events,
                    self._tab_newplayers,
                    self._tab_players,
                    self._tab_ecfgradingcodes,
                    self._tab_ecfclubcodes,
                    self._tab_ecfplayers,
                    self._tab_ecfevents,
                    ),
                self._state_ecfeventdetail: (
                    self._tab_ecfeventdetail,
                    ),
                self._state_importecfdata: (
                    self._tab_importecfdata,
                    ),
                self._state_importfeedback: (
                    self._tab_importfeedback,
                    ),
                self._state_joineventplayers: (
                    self._tab_joineventplayers,
                    ),
                },
            switch_state={
                (self._state_dbopen,
                 ecfevents.ECFEvents._btn_ecfeventdetail
                 ): [self._state_ecfeventdetail, self._tab_ecfeventdetail],
                (self._state_ecfeventdetail,
                 newevent.NewEvent._btn_ok
                 ): [self._state_dbopen, self._tab_ecfevents],
                (self._state_ecfeventdetail,
                 newevent.NewEvent._btn_cancel
                 ): [self._state_dbopen, self._tab_ecfevents],
                (self._state_dbopen,
                 control.Control._btn_copyecfmasterplayer
                 ): [self._state_importecfdata, self._tab_importecfdata],
                (self._state_dbopen,
                 control.Control._btn_copyecfmasterclub
                 ): [self._state_importecfdata, self._tab_importecfdata],
                (self._state_dbopen,
                 control.Control._btn_copyecfupdateplayer
                 ): [self._state_importecfdata, self._tab_importecfdata],
                (self._state_dbopen,
                 control.Control._btn_copyecfupdateclub
                 ): [self._state_importecfdata, self._tab_importecfdata],
                (self._state_dbopen,
                 control.Control._btn_copyecfnewplayer
                 ): [self._state_importecfdata, self._tab_importecfdata],
                (self._state_importecfdata,
                 importecfdata.ImportECFData._btn_closeecfimport
                 ): [self._state_dbopen, self._tab_control],
                (self._state_dbopen,
                 control.Control._btn_ecfresultsfeedback
                 ): [self._state_importfeedback, self._tab_importfeedback],
                (self._state_importfeedback,
                 feedback.Feedback._btn_closefeedback
                 ): [self._state_dbopen, self._tab_control],
                (self._state_dbopen,
                 events.Events._btn_join_event_new_players
                 ): [self._state_joineventplayers, self._tab_joineventplayers],
                (self._state_joineventplayers,
                 joineventplayers.JoinEventPlayers._btn_cancel
                 ): [self._state_dbopen, self._tab_events],
                },
            )
        
    def get_ecf_event_detail_context(self):
        """Return the ECF event page."""
        return self.get_tab_data(self._tab_ecfevents)

    def set_ecfdataimport_module(self, enginename):
        """Import the ECF reference data import module."""
        self._ecfdataimport_module = importlib.import_module(
            ECF_DATA_IMPORT_MODULE[enginename], 'results.gui')

    def set_knownnamesdatasource_module(self, enginename):
        """Import the known names datasource module."""
        self._knownnamesdatasource_module = importlib.import_module(
            KNOWN_NAME_DATASOURCE_MODULE[enginename], 'results.gui')
